/***********************************************************************
 Name             : Matrix.java
 Autor            : Marcel Lattmann
                    Eidgenoessisch Technische Hochschule Zuerich (ETHZ)
 Letzte Aenderung : 29.7.1999

 Algorithmen fuer lineare Algebra auf (dichte) Matrizen. Ein kleiner
 Parser kann aus einem Textinput eine Matrix erzeugen. 
 
 Still to do:       exceptions.
                    Funktioniert addRows / addColumns korrekt?

 Benoetigt folgende Klassen:
   keine

 History:
   26.7.1999: Uebernommen von Bryan Lewis von der Kent State University, 
              Department of Mathematics & Computer Science
              blewis@mcs.kent.edu    http://www.mcs.kent.edu/~blewis/

************************************************************************/

import java.util.Vector;
import java.lang.Math;

public class Matrix {
  
  public int zeilen, spalten;
  public double[][] element;	// Array, das die Elemente enthaelt
  
  public Matrix() {
    // Erzeuge eine leere Matrix
    zeilen = 0;
    spalten = 0;
    //element = new double[zeilen][spalten];
  }

  public Matrix(int r, int c) {
    //  Erzeuge eine leere r mal c Matrix.
    zeilen = r;
    spalten = c;
    element = new double[zeilen][spalten];
  }
  
  public Matrix(double d) {
    //  Erzeuge eine 1x1-Matrix. Der Wert des Elementes ist d
    zeilen = 1;
    spalten = 1;
    element = new double[1][1];
    element[0][0] = d;
  }

  public Matrix(int z, int s, double fill) {
    //  Erzeuge eine z mal s-Matrix mit den Werten 'fill'
    zeilen = z;
    spalten = s;
    element = new double[zeilen][spalten];
    int i, j;
    for (i=0; i<zeilen; i++) {
      for (j=0; j<spalten; j++) {
	element[i][j] = fill;
      }
    }
  }

  public Matrix(Matrix m) {
    //  Erzeuge eine neue Kopie von m
    zeilen = m.zeilen;
    spalten = m.spalten;
    element = new double[zeilen][spalten];
    int i, j;
    for (i=0; i<zeilen; i++) {
      for (j=0; j<spalten; j++) {
	element[i][j] = m.element[i][j];
      }
    }
  }

  public Matrix(int z, int s, char code) {
    // Erzeuge einen spezielle Matrix der Groesse z mal s
    // Code = 'I' : Identitaet
    //        'H' : Hilbert
    //        'R' : Zufaellig (random) uniform in [0,1]
    zeilen = z;
    spalten = s;
    element = new double[zeilen][spalten];
    int i, j;
    if ((code == 'i') || (code == 'I')){
      // Identitaetsmatrix
      for (i = 0; i < zeilen; i++)
	if (i < spalten)
	  element[i][i] = 1;
    }
    else if ((code == 'h') || (code == 'H')){
      // Hilbert Matrix
      for (i = 0; i < zeilen; i++)
	for (j=0; j<spalten; j++)
	  element[i][j] = 1/((double)i+(double)j+1);
    }
    else if ((code == 'r') || (code == 'R')){
      // Zufaellige Matrix mit Elementen uniform in [0,1]
      for (i = 0; i < zeilen; i++)
	for (j=0; j<spalten; j++)
	  element[i][j] = Math.random();
    }
  }

  public Matrix(String s) {
    /* Erzeugt ein Matrix Objekt und parst den String in
       das Elementarray. Das Format des Strings kann
       Matlab-like sein oder folgendes Format haben:
       spalten durch Tabs, Kommas oder Spaces getrennt
       zeilen durch Newline oder Semikolon getrennt
       Zeilen, die mit Spaces gefuellt sind, werden gekuerzt
    */

    Vector row = new Vector();	// data will be assembled into these vectors
    Vector col = new Vector();	// and then transferred 
                                // into the array element[][].    
    s = s + " ;";
    int i = s.length();
    int j;
    int rowCounter = 0;
    int colCounter = 0;
    String sData = new String(); // will hold each element during parsing
    Double fl;
    char sChar;
    for (j = 0; j<i; j++) {
      sChar = s.charAt(j);
      // check for a delimiter...
      if ( (sChar==' ')|| (sChar==',')|| ( (int) sChar==9)||
	   (sChar == ';')||( (int) sChar == 13)||( (int) sChar == 10) ) {
	fl = new Double(0);
	// See if the string in sData represents a number...
	try {
	  boolean testSpace = true;
	  int ii;
	  for(ii=0;ii<sData.length();ii++){
	    testSpace=testSpace&&(sData.charAt(ii)==' '); }
	  if(testSpace==false){
	    fl = new Double(sData);
	    col.addElement(sData); }	// append column element as string
	  sData = new String();	// wipe out contents of string
	} 
	catch (Exception e) {
	// non-numeric stuff...
	  sData = new String();	// wipe out contents of string
	}	
	if ( ( (sChar == ';')||( (int) sChar == 13)||( (int) sChar == 10) ) &&
	     !col.isEmpty() ) {
	  row.addElement(col); // append row (i.e., vector of column elements)
	  rowCounter = rowCounter + 1;
	  sData = new String();		// wipe out contents of string
	  colCounter = col.size();
	  col = new Vector();	// wipe out the column vector
	  /* an interesting Java (1.02) note: use new Vector() method to
	     force the contents of this vector to be explicitly copied
	     into the row vector. The removeAllElements method will not
	     work in this situation (try it!).
	  */
	}
      }
      // build up data...
      else {
	if ((Character.isDigit(sChar))||(sChar=='.')||(sChar=='-')) {
	  // allow only digit and decimal point characters
	  sData = sData + sChar;	// append to string
	}
      }
      
    }
    zeilen = rowCounter;
    spalten = colCounter;
    element = new double[zeilen][spalten];
    col = new Vector();
    Double d = new Double(0);
    for (j = 0; j<zeilen; j++) {
      col = (Vector) row.elementAt(j);
      for (i = 0; i<col.size(); i++) {
	d = new Double((String)col.elementAt(i));
	element[j][i] = d.doubleValue();
      }
    }
  }	

  public Matrix transpose() {
    // returns the transpose of this matrix object
    Matrix t = new Matrix(spalten, zeilen);
    int i, j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	t.element[j][i] = this.element[i][j];
      }
    }
    return t;
  }

  public Matrix diag() {
    /* Returns the diagonal of this matrix (as a vector), or
       if given a vector, returns a diagonal matrix
    */
    Matrix t = new Matrix(zeilen, 1);
    if(spalten==1){
      t = new Matrix(zeilen, zeilen);
    }
    int i, k, j;
    for (i = 0; i<zeilen; i++) {
      if(spalten>1){
	k=0; j=i;
      }
      else{
	k=i; j=0;
      }
      t.element[i][k] = this.element[i][j];
    }
    return t;
  }

  public static Matrix add(Matrix m1, Matrix m2){
    // Return the matrix m = m1 + m2
    Matrix m=new Matrix(m1.zeilen,m1.spalten);
    if ((m1.zeilen == m2.zeilen)&&(m1.spalten==m2.spalten)) {
      int i,j;
      for (i=0; i<m.zeilen; i++) {
	for (j=0; j<m.spalten; j++) {
	  m.element[i][j] = m1.element[i][j] + m2.element[i][j]; 
	}
      }
    }
    return m;
  }

  public static Matrix subtract(Matrix m1, Matrix m2){
    // Return the matrix m = m1 - m2
    Matrix m=new Matrix(m1.zeilen,m1.spalten);
    if ((m1.zeilen == m2.zeilen)&&(m1.spalten==m2.spalten)) {
      int i,j;
      for (i=0; i<m.zeilen; i++) {
	for (j=0; j<m.spalten; j++) {
	  m.element[i][j] = m1.element[i][j] - m2.element[i][j]; 
	}
      }
    }
    return m;
  }

  public static Matrix multiply(double d, Matrix m1){
    // Return the matrix m = d*m1
    Matrix m=new Matrix(m1.zeilen,m1.spalten);
    int i,j;
    for (i=0; i<m.zeilen; i++) {
      for (j=0; j<m.spalten; j++) {
	m.element[i][j] = d * m1.element[i][j];
      }
    }
    return m;
  }

  public static Matrix multiply(Matrix m1, Matrix m2){
    /* Matrix-Matrix or Matrix-vector product
       returns m=m1*m2
       m1 can be a 1x1 Matrix for scalar-Matrix product
    */
    Matrix m = new Matrix(0);
    if (m1.spalten == m2.zeilen) {
      // matrix product
      double sum = 0;
      int k = 0;
      m = new Matrix(m1.zeilen,m2.spalten);
      int i,j;
      for (i=0; i<m.zeilen; i++) {
	for (k=0; k<m2.spalten; k++) {
	  for (j=0; j<m1.spalten; j++) {
	    sum = sum + m1.element[i][j] * m2.element[j][k];
	  }
	  m.element[i][k] = sum;
	  sum = 0;
	}
      }
    }
    else if ((m1.spalten == 1)&&(m1.zeilen == 1)) {
      // scalar-vector product
      m = new Matrix(m2.zeilen,m2.spalten);
      int i,j;
      for (i=0; i<m.zeilen; i++) {
	for (j=0; j<m.spalten; j++) {
	  m.element[i][j] = m1.element[0][0] * m2.element[i][j];
	}
      }
    }
    return m;
  }

  public static Matrix divide(Matrix m1, Matrix m2) {
    /* Returns m1/m2. If m2 is a 1x1 matrix, then this is
       just matrix/scalar. If m2 is a square, invertible
       matrix and m1 is a vector (a matrix with one column), 
       divide returns inverse(m2)*m1, using the
       Householder QR algorithm.
    */
    Matrix m = new Matrix(0);
    if ((m2.spalten == 1)&&(m2.zeilen == 1)) {
      // vector-scalar division
      m=new Matrix(m1.zeilen,m1.spalten);
      int i,j;
      for (i=0; i<m.zeilen; i++) {
	for (j=0; j<m.spalten; j++) {
	  m.element[i][j] = m1.element[i][j] / m2.element[0][0];
	}
      }
    }
    else if ((m2.spalten == m2.zeilen)&&
	     (m1.spalten == 1)&&(m1.zeilen == m2.zeilen)) {
      /* Solve a general, dense, non-singular linear 
	 system Ax=b via QR, where A=m2, b=m1, and x is returned. */
      m=new Matrix(m2.zeilen,1);
      Matrix Q=m2.Q();
      Matrix R=m2.R();
      Matrix b=multiply(Q.transpose(),m1);
      int i,j;
      double sum = 0;
      m.element[m.zeilen-1][0] =
	b.element[m.zeilen-1][0]/R.element[m.zeilen-1][m.zeilen-1];
      i=m.zeilen-1;
      while(i >= 0) {
	sum = 0;
	j = m.zeilen-1;
	while(j>=i+1) {
	  sum = sum + R.element[i][j]*m.element[j][0];
	  j--;
	}
	m.element[i][0] = (b.element[i][0]-sum)/R.element[i][i];
	i--;
      }
    }
    return m;
  }

  public Matrix sub(int r1, int r2, int c1, int c2) {
    // returns the submatrix (r1:r2,c1:c2) (Moeler notation)
    // requires r2>=r1, c2>=c1
    Matrix A = new Matrix(r2 - r1 + 1, c2 - c1 + 1);
    int i, j;
    for (i = r1; i<=r2; i++) {
      for (j = c1; j<=c2; j++) {
	A.element[i - r1][j - c1] = this.element[i][j];
      }
    }
    return A;
  }
  
  public Matrix appendCols(Matrix x){
    // append the column vectors in x to this matrix
    Matrix M=new Matrix(zeilen,spalten+x.spalten);
    int i,j;
    for(i=0;i<x.zeilen;i++){
      for (j=0;j<spalten;j++){
	M.element[i][j]=this.element[i][j];
      }
      for(j=0;j<x.spalten;j++){
	M.element[i][spalten+j]=x.element[i][j];
      }
    }
    M.zeilen  = zeilen;
    M.spalten = spalten+1;
    return M;
  }
  
  public Matrix appendRows(Matrix x){
    // append the row vectors in x to this matrix
    Matrix M=new Matrix(zeilen+x.zeilen,spalten);
    int i,j;
    for(i=0;i<x.spalten;i++){
      for(j=0;j<zeilen;j++){
	M.element[j][i]=this.element[j][i];
      }
      for(j=0;j<x.zeilen;j++){
	M.element[zeilen+j][i]=x.element[j][i];
      }
    }
    M.zeilen  = zeilen+1;
    M.spalten = spalten;
    return M;
  }

  public Matrix deleteRow(int z){
    // loescht die Zeile z der Matrix
    Matrix M=new Matrix(zeilen-1,spalten);
    int i,j;
    for(i=0;i<spalten;i++){
      for(j=0;j<z;j++){
	M.element[j][i]=element[j][i];
      }
      for(j=z+1;j<zeilen;j++){
	M.element[j-1][i]=element[j][i];
      }
    }
    M.zeilen  = zeilen-1;
    M.spalten = spalten;
    return M;
  }

  public Matrix flipud(){
    // flip this matrix vertically
    Matrix M=new Matrix(zeilen,spalten);
    int k,j;
    for(k=0;k<zeilen;k++){
      for(j=0;j<spalten;j++){
	M.element[zeilen-k-1][j]=this.element[k][j];
      }
    }
    return M;
  }

  public Matrix fliplr(){
    // flip this matrix horizontally
    Matrix M=new Matrix(zeilen,spalten);
    int k,j;
    for(k=0;k<zeilen;k++){
      for(j=0;j<spalten;j++){
	M.element[k][spalten-j-1]=this.element[k][j];
      }
    }
    return M;
  }

  public Matrix permute(int a1, int a2, char c) {
    /*	Returns a permuted matrix according  code c
	where c is in {'c', 'r'} for spalten or zeilen and
	a1, a2 represent the spalten/zeilen to swap
    */
    Matrix p = new Matrix(this);
    int i, j;
    if (c == 'r') {
      for (i=0; i<spalten; i++) {
	p.element[a1][i] = this.element[a2][i];
	p.element[a2][i] = this.element[a1][i];
      }
    }
    else if (c == 'c') { 
      for (i=0; i<zeilen; i++) {
	p.element[i][a1] = this.element[i][a2];
	p.element[i][a2] = this.element[i][a1];
      }
    }
    return p;
  }

  public double norm() {
    /* returns the Frobenius norm (Matrix), or Euclidean norm (Vector)
       This is the default norm for a Matrix object. Use the Norm
       class for different norms.
    */
    double l = 0;
    int i, j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	l = l + this.element[i][j] * this.element[i][j];
      }
    }
    l = Math.pow(l, 0.5);
    return l;
  }
	
  public double max() {
    // Gibt das positivste Element der Matrix oder des Vektors retour
    double m = this.element[0][0];
    int i,j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	if(this.element[i][j] > m){
	  m = this.element[i][j];
	}
      }
    }
    return m;
  }
	
  public double min() {
    // Gibt das negativste Element der Matrix oder des Vektors retour
    double m = this.element[0][0];
    int i,j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	if(this.element[i][j] < m){
	  m = this.element[i][j];
	}
      }
    }
    return m;
  }

  public double sum() {
    /* returns the sum of all the elements in the matrix or vector
     */
    double s = 0;
    int i, j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	s = s + this.element[i][j];
      }
    }
    return s;
  }
	
  public double average() {
    // returns the average of all the elements in the matrix or vector
    double s = 0;
    int i, j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	s = s + this.element[i][j];
      }
    }
    return s/(spalten*zeilen);
  }
  
  public double sumSquares() {
    // returns the sum of the squares 
    // of all the elements in the matrix or vector
    double s = 0;
    int i, j;
    for (i = 0; i<zeilen; i++) {
      for (j = 0; j<spalten; j++) {
	s = s + Math.pow(this.element[i][j],2);
      }
    }
    return s;
  }
	
  public Matrix  Q() {
    /*	returns the 'Q' in the QR-decomposition of this matrix object
	using Householder reflections, without column pivoting
    */
    Matrix P = new Matrix(zeilen, zeilen, 'I');
    Matrix A = new Matrix(this);
    Matrix AA, PP;
    int i, j;
    Matrix v;
    
    for(j = 0; j<spalten; j++) {
      v = A.sub(0,A.zeilen-1, j, j);
      if (j>0) {
	for (i = 0; i<j; i++) {
	  v.element[i][0] = 0;
	}
      }
      v.element[j][0] = v.element[j][0] + v.norm() * sign(v.element[j][0]);
      double r = (double) -2 / (v.norm() * v.norm() );
      AA = new Matrix(A);
      A = multiply(v.transpose(),A);
      A = multiply(v,A);
      A = multiply(r,A);
      A = add(AA,A);
      PP = new Matrix(P);
      P = multiply(v.transpose(),P);
      P = multiply(v,P);
      P = multiply(r,P);
      P = add(PP,P);
    }
    return P.transpose();
  }
  
  public Matrix  R() {
    /*	returns the 'R' in the QR-decomposition of this matrix object
	using Householder reflections, without column pivoting
    */
    Matrix P = new Matrix(zeilen, zeilen, 'I');
    Matrix A = new Matrix(this);
    Matrix AA, PP;
    int i, j;
    Matrix v;
    
    for(j = 0; j<spalten; j++) {
      v = A.sub(0,A.zeilen-1, j, j);
      if (j>0) {
	for (i = 0; i<j; i++) {
	  v.element[i][0] = 0;
	}
      }
      v.element[j][0] = v.element[j][0] + v.norm() * sign(v.element[j][0]);
      double r = (double) -2 / (v.norm() * v.norm() );
      AA = new Matrix(A);
      A = multiply(v.transpose(),A);
      A = multiply(v,A);
      A = multiply(r,A);
      A = add(AA,A);
      PP = new Matrix(P);
      P = multiply(v.transpose(),P);
      P = multiply(v,P);
      P = multiply(r,P);
      P = add(PP,P);
    }
    return A;
  }

  public Vector  qr() {
    /*	returns the QR-decomposition of this matrix object
	using Householder reflections, without column pivoting
	The results is a (Java) vector containing {Q, R}
    */
    Vector result = new Vector();
    Matrix P = new Matrix(zeilen, zeilen, 'I');
    Matrix A = new Matrix(this);
    Matrix AA, PP;
    int i, j;
    Matrix v;
    
    for(j = 0; j<spalten; j++) {
      v = A.sub(0,A.zeilen-1, j, j);
      if (j>0) {
	for (i = 0; i<j; i++) {
	  v.element[i][0] = 0;
	}
      }
      v.element[j][0] = v.element[j][0] + v.norm() * sign(v.element[j][0]);
      double r = (double) -2 / (v.norm() * v.norm() );
      AA = new Matrix(A);
      A = multiply(v.transpose(),A);
      A = multiply(v,A);
      A = multiply(r,A);
      A = add(AA,A);
      PP = new Matrix(P);
      P = multiply(v.transpose(),P);
      P = multiply(v,P);
      P = multiply(r,P);
      P = add(PP,P);
    }
    result.addElement(A);      // R (at element 0)
    result.addElement(P.transpose());	// Q (at element 1)
    return result;
  }

  public Vector  toHess() {
    /*	makes the matrix upper Hessenberg via Householder reflections
	returns  {P, H} s.t. P' * this * P = H and H is upper Hessenberg
	and P' * P = I.
    */
    Vector result = new Vector();	// the result
    Matrix P = new Matrix(zeilen, spalten, 'I');
    Matrix I = new Matrix(zeilen, spalten, 'I');
    Matrix A = new Matrix(this);
    int i, j, k;
    Matrix v;
    
    for(j = 0; j<spalten-2; j++) {
      v = new Matrix(zeilen, 1);
      v.element[j][0] = 1;
      v = multiply(A,v); // get the j-th column
      for (i=0; i<(j+1); i++) {
	v.element[i][0] = 0;
      }
      v.element[j+1][0] = v.element[j+1][0] + 
	v.norm() * sign(v.element[j+1][0]);
      
      double r = (double) -2 / (v.norm() * v.norm());
      v = multiply(v,v.transpose());
      v = multiply(r,v);
      v = add(I,v);
      P = multiply(P,v);
      A = multiply(P.transpose(),this);
      A = multiply(A,P);
    }
    
    result.addElement(P);   // the orthogonal transformation
    result.addElement(A);   // the upper-Hessenberg form
    return result;
  }

  public Vector  gepp() {

    /*	returns the LU decomposition of a matrix using the Gauss
	transform. This algorithm returns 3 matrices as follows:
	{P, L, U} such that LU = PA. This algorithm performs partial
	pivoting.
	Written 3-March, 1997 by Bryan Lewis
    */
    Vector v = new Vector();	// the result
    Matrix P = new Matrix(zeilen, spalten, 'I');	
    // P will track the permutations
    Matrix L = new Matrix(zeilen, spalten, 'I');	// the lower triangle
    Matrix U = this;	// this matrix to be transformed to upper triangular
    Matrix G = new Matrix(zeilen, spalten, 'I');	
    // temporary Gauss transform matrix
    int i, j, k, p;
    double d;
    
    for (j = 0; j < spalten-1; j++) {
      // start of parital pivot code:
      d = Math.abs(U.element[j][j]);
      p = j;
      for (i = j+1; i < zeilen; i++) {
	if (Math.abs(U.element[i][j]) > d) {
	  // System.out.println(U.element[i][j] +", "+i);
	  d = Math.abs(U.element[i][j]);
	  p = i;
	}
      }
      if (p > j) {
	U = U.permute(j, p, 'r');
	P = P.permute(j, p, 'r'); // don't forget to track permutations
      }
      // end of partial pivot code.
      for (i = j+1; i < zeilen; i++) {
	if (U.element[j][j]!=0) {
	  G.element[i][j] = -U.element[i][j]/U.element[j][j];
	}
      }
      U = multiply(G, U);
      L = L.permute(j, p, 'r');
      for (k = 0; k < j; k++) {
	L.element[k][j] = 0;
      }
      L.element[j][j] = 1;
      for (k = j+1; k < zeilen; k++) {
	L.element[k][j] = -G.element[k][j];
	G.element[k][j] = 0;
      }
      
    }
    for (k = 0; k < zeilen; k++) {
      L.element[k][spalten-1] = 0;
    }
    L.element[zeilen-1][spalten-1] = 1;
    
    v.addElement(P);
    v.addElement(L);
    v.addElement(U);
    return v;
  }

  public double leig(double p) {
    /*	Elementary QR  method method to find the spectral radius of
	a positive valued matrix. Parameter p = precision desired.
	For example, if A is a Matrix of  positive real numbers, then
	A.leig(0.01) returns the largest eigenvalue to at least two
	digits of accuracy.
    */
    Vector qr;
    Matrix Q = new Matrix(zeilen, spalten);
    Matrix R = new Matrix(zeilen, spalten);
    Matrix A = new Matrix(this);	// initialized
    int i = 1;
    int maxIter = 200-this.zeilen;
    if(maxIter<25){maxIter=25;} // set up a maximum iteration count
    double v = 99;		// temporary result
    double res = 99;	// residual
    while((i<maxIter)&&(res>p)) {
      qr = A.qr();
      Q = (Matrix)qr.elementAt(1);
      R = (Matrix)qr.elementAt(0);
      A = multiply(R,Q);
      i++;
      res = Math.abs(A.element[0][0] - v);
      v = A.element[0][0];
    }
    return A.element[0][0];
  }

  public Matrix  sort() {
    /* Sort this object in the following way:
       1. If this is a column or row vector, sort the vector
          and return a two column Matrix containing the sorted
	  vector in the first column, and the permutation indices
	  in the 2nd column.
       2. If this is a rectangular Matrix, sort the zeilen according
          to the entries in the first spalte.
    */
    Matrix R = new Matrix();
    int i,k;
    double d=Math.max(spalten,zeilen);
    int lngth = (int)d;
    double a[] = new double[lngth];
    int indx[] = new int[lngth];

    if((zeilen == 1)||(spalten==1)) {
      a = new double[lngth];	// contains the data for sorting
      indx = new int[lngth];	// index array
      for (i=0; i<lngth; i++) {
	if(zeilen<spalten){
	  a[i] = element[0][i];}
	else {
	  a[i] = element[i][0];}
	indx[i] = i + 1;
      }
      qsort(a, indx, 0, lngth - 1);
      R = new Matrix(lngth, 2);
      for (i=0; i<lngth; i++) {
	R.element[i][0] = a[i];
	R.element[i][1] = indx[i];
      }
    }
    else if (spalten > 1) {
      // sort the array by the first column
      a = new double[zeilen];	// contains the data for sorting
      indx = new int[zeilen];	// index array
      for (i=0; i<zeilen; i++) {
	a[i] = element[i][0];
	indx[i] = i + 1;
      }
      qsort(a, indx, 0, zeilen - 1);
      R = new Matrix(zeilen, spalten);
      for(i=0; i<zeilen; i++) {
	for(k=0; k<spalten; k++){
	  R.element[i][k] = element[indx[i]-1][k];
	}
      }
    }
    return R;
  }
	
  public Matrix  order() {
    /*	Sorts this vector, returning a ranked vector,
	(here, 'vector' indicates a 1-d Matrix, not the Java class Vector).
	'order' is used so as to not confuse with the usual definition of rank
    */
    Matrix S = this.sort();
    Matrix y = new Matrix(S.zeilen, 1);
    double[] v = new double[S.zeilen];
    // the only trick here is to handle ties!
    int i = 0, k, j, l;
    while (i<S.zeilen){
      j = 0; l = 0;
      for(k=i;k<S.zeilen;k++){
	if(S.element[k][0]==S.element[i][0]) {
	  j = j + 1;
	  l = l + k +1;
	}
      }
      for(k=0;k<j;k++){
	v[i+k]=(double)(((double)(l))/(double)j);
      }
      i = i + j;
    }
    // now unsort v and return it...
    for (i=0;i<S.zeilen; i++){
      y.element[(int)S.element[i][1]-1][0]=v[i];
    }
    return y;
  }
  
  // The following methods are used internally by the Matrix class:
	
  double sign(double d) {
    // returns the sign of the supplied double-precision argument
    double s = 1;
    if (d<0) { s = -1; }
    return s;
  }
	
  void qsort(double a[], int index[], int lo0, int hi0)   {
    /* Recursive Quick Sort algorithm
       a[]=input 1-d array
       index[]=output permutation array
       lo0=input lower index
       hi0=input upper index
    */
      int lo = lo0;
      int hi = hi0;
      double mid;
 
      if ( hi0 > lo0)
	{
	  mid = a[ ( lo0 + hi0 ) / 2 ];
	  while( lo <= hi )
	    {
	      while( ( lo < hi0 ) && ( a[lo] < mid ) )
		++lo;
	      while( ( hi > lo0 ) && ( a[hi] > mid ) )
		--hi;
	      if( lo <= hi ) 
		{
		  swap(a, lo, hi);
		  swap(index, lo, hi);
		  ++lo;
		  --hi;
		}
	    }
	  if( lo0 < hi )
            qsort( a,index, lo0, hi );
	  if( lo < hi0 )
            qsort( a,index, lo, hi0 );
	}
  }

  private void swap(double a[], int i, int j){
    double T;
    T = a[i]; 
    a[i] = a[j];
    a[j] = T;
  }
  private void swap(int a[], int i, int j){
    int T;
    T = a[i]; 
    a[i] = a[j];
    a[j] = T;
  }
}

