/***********************************************************************
 Name             : ed50.java
 Autor            : Marcel Lattmann
                    Eidgenoessisch Technische Hochschule Zuerich (ETHZ)
 Letzte Aenderung : 28.7.1999

 Einfaches Applet zur Berechnung des ED50-Wertes
 
 Benoetigt folgende Klassen:
   edFrame.class         "Hauptprogramm" des Applets
   Matrix.class	          Lineare Algebra mit Matrizen
   ExampleData.class      Beispieldaten erzeugen oder laden
   Plot.class             Kurve plotten
   plotText.class         Text innerhalb der Kurve ausgeben
	
 History:
   26.7.1999     Erster Enwtwurf
   28.7.1999     Plotten, Skalieren eingebaut
    4.8.1999     Printen der Titeldaten

************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.applet.*;
import java.io.*;

public class ed50Main{
 public static void main(String argv[]) {
    ed50Frame f = new ed50Frame(600,600,"ED50",Color.lightGray);
  }
}
class ed50Frame extends Frame {

  public static final int NRITEMS = 8;

  protected TextArea data;		// data entry field
  protected TextArea result;	        // output field
  protected Plot plot = new Plot();
  protected Button compute, header;
  protected Label dataLabel, resultLabel;
  protected Label titelLabel[] = new Label[NRITEMS+2];
  protected Label textLabel[] = new Label[NRITEMS+2];

  Color background;
  public String feldText[] = new String[8];
  private String feldTitel[]={"Titel","Pilz","Stmme","Referenzstmme","Test-Fungizide","Versuchsvariante","Versuchsperson","Bemerkungen","",""};

  int fontSize = 12;
  Font defaultFont = new Font("COURIER", Font.PLAIN, fontSize);

  sigmoid s = new sigmoid();

  int i, offy; 
  double zero=0.00001;  // precision for output display
  double x,stepx;

  String newln;
  String res;

  private void placeLabel(Label l, GridBagConstraints c, int x, int w, int y) {
    l.setBackground(background);
    c.gridx = x; c.gridy = y;
    c.gridwidth  = w;
    c.gridheight = 1;
    c.weightx = c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.NORTHWEST;
    this.add(l,c);
  }

  public ed50Frame(int width, int height, String title, Color background) {
    super(title);
    setSize(width, height);
 
    int offy=0;

    // Set line termination character
    newln = System.getProperty("line.separator");

    dataLabel = new Label("Data");
    dataLabel.setBackground(background);

    data = new TextArea("", 20, 20);
    data.setBackground(background);
    data.setFont(defaultFont);

    plot.clear(true);
    plot.setBackground(background);
    plot.setBars(false);
    plot.setXLog(true);
    plot.setYLog(false);
    plot.setXLabel("Konzentration [mg/l]");
    plot.setYLabel("Gemessene Hemmung");
    plot.setMarksStyle("dots",1);
    plot.setMarksStyle("none",2);
    plot.setTitle("Approximation der Kurve");
    plot.read();
    plot.setSize(700,600);

    resultLabel = new Label("Results");
    resultLabel.setBackground(background);

    result = new TextArea("The results will appear here",6,10);
    result.setBackground(background);
    result.setFont(defaultFont);

    compute = new Button("Compute");
    compute.addActionListener(new computeListener());
    compute.setBackground(background);

    header = new Button("Header");
    header.addActionListener(new headerListener());
    header.setBackground(background);

    setBackground(background);

    GridBagConstraints c = new GridBagConstraints();
    GridBagLayout gb = new GridBagLayout();

    setLayout(gb);

    // Schreibe Titelinformation
    // System.out.println("paint");
   
    for (i = 0; i < NRITEMS+2; i++) {
      titelLabel[i] = new Label(feldTitel[i]);
      placeLabel(titelLabel[i],c,0,1,i);
      textLabel[i] = new Label("                                                                                                                                 ");
      placeLabel(textLabel[i],c,1,GridBagConstraints.REMAINDER,i); 
    }

    offy=NRITEMS+3;

    c.gridx = 0; c.gridy = offy;
    c.gridwidth  =  GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    c.weightx = c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.SOUTHWEST;
    add(dataLabel,c);

    c.gridx = 0; c.gridy = 1+offy;
    c.gridwidth  = 3;
    c.gridheight = 20;
    c.weightx = 0.0; c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.NORTHWEST;
    add(data,c);

    c.gridx = 3; c.gridy = 1+offy;
    c.gridwidth  = GridBagConstraints.REMAINDER;
    c.gridheight = 20;
    c.weightx = 1.0; c.weighty = 1.0;
    c.fill = GridBagConstraints.BOTH;
    c.anchor = GridBagConstraints.CENTER;
    add(plot,c);

    c.gridx = 0; c.gridy = 21+offy;
    c.gridwidth  = GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    c.weightx = c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.SOUTHWEST;
    add(resultLabel,c);

    c.gridx = 0; c.gridy = 22+offy;
    c.gridwidth  = GridBagConstraints.REMAINDER;
    c.gridheight = 8;
    c.weightx = 0.0; c.weighty = 0.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.anchor = GridBagConstraints.CENTER;
    add(result,c);

    c.gridx = 0; c.gridy = 30+offy;
    c.gridwidth  = 5;   //GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    c.weightx = c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.CENTER;
    add(compute,c);

    c.gridx = 5; c.gridy = 30+offy;
    c.gridwidth  = GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    c.weightx = c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.CENTER;
    add(header,c);

    addWindowListener(new wAdapter());

    // setSize(720,760);
    this.setVisible(true);
  }

  String monoString(String s) {
    // convert a generic string s to a fixed length one	
    String sAdd = new String(s + "                                      ");
    return sAdd.substring(0,14);
  }
  String monoString(double s) {
    // convert a generic double s to a "nice" fixed length string	
    Double sD = new Double(s);
    String sAdd = new String();
    if(s>zero){
      sAdd = new String(sD.toString()); }
    else{ sAdd = "<0.00001"; }
    if(sAdd.length()>10){
      sAdd = sAdd.substring(0,10); }
    sAdd = sAdd + "                                      ";
    return sAdd.substring(0,14);
  }
  String monoString(int s) {
    // convert a generic integer s to a fixed length string	
    Integer sD = new Integer(s);
    String sAdd = new String(sD.toString());
    sAdd = sAdd + "                                      ";
    return sAdd.substring(0,14);
  }  

  //public void actionPerformed(ActionEvent e) {
    // / h.getData(background,defaultFont);
    //  
    //}

  class headerListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {

      Frame f = new Frame();

      // Create an instance of InfoDialog, with title and message specified
      Header h = new Header(f, feldText, defaultFont, Color.lightGray);
      h.show();
      feldText = h.getData();    
      for (int i =0;i<NRITEMS+2;i++) {
        ed50Frame.this.textLabel[i].setText(feldText[i]);
      }
    }
  }
  /* Eventhandler fuer den Compute-Button als Inner class von
     ed50 */
  class computeListener implements ActionListener {

    public void actionPerformed(ActionEvent e) {
      // calculate the result
      s.P = new Matrix(data.getText());;

      if (s.P.zeilen ==0) {
         result.setText("Uncorrect input.");
         return;
      }       
      res= s.checkData(newln);
      if (res.length() > 0) {
         result.setText(res);
         return;
      }
      if (s.P.zeilen < 4) {
         result.setText("Not enough data points for fit.");
         return;
      }
      // calculate the result
      result.setText("Calculating...");
      s.fit(newln);

      res = s.res+ "Results: ED50 = " + monoString(s.ed50) +
            "    ED95 = " + monoString(s.ed95) + newln +
            "Quality of approximation = " + monoString(s.sumMin) +
            "    Number of iterations = " + monoString(s.iter) + newln +
            "Curve Parameters: a = " + monoString(s.a) +
            "   b = " + monoString(s.b) + newln;

      /* Alten Plot loeschen. sofern vorhanden.
         Kontrollinformation noch behalten, daher Argument false. */
      plot.clear(false);
      plot.setXRange(s.Xmin(true)-0.5,s.Xmax(true));  // true -> log10(X)
      plot.setYRange(s.Ymin(false)-0.05,s.Ymax(false)+0.05);
      // Messpunkte plotten
      for (i=0; i<s.P.zeilen; i++) {
        plot.addPoint(1, s.P.element[i][0],s.P.element[i][1],false);
      }
   
      // Sigmoid plotten
      stepx=Math.log( s.P.element[s.P.zeilen-1][0]/ s.P.element[0][0])/50;
      stepx=Math.exp(stepx);

      x = s.P.element[0][0];
      // Den ersten Punkt nicht mit vorhergehendem verbinden
      plot.addPoint(2, x,s.f(x),false);

      // Die weiteren Punkte mit den vorhergehenden verbinden.
      while (x < s.P.element[s.P.zeilen-1][0]) {
        x = x*stepx;
        plot.addPoint(2, x,s.f(x),true);
      }    
      plot.repaint();
      result.setText(res);
    }
  }	

  //***********************************************************************
  // Interne Klasse, um die WindowEvents zu handeln

  class wAdapter extends WindowAdapter {
    public void windowClosing(WindowEvent e) {	
      dispose();
      System.exit(0);
    }
  }
}
